/*  Project image
		AutoMedia Ltd.
		Copyright 1997. AutoMedia Ltd.

		SUBSYSTEM:    imglib.apx Dinamic Library
		FILE:         msttsdlg.cpp
		AUTHOR:       Bidus Yura


		OVERVIEW
		========
		Source file for implementation of MStatusDlg (TDialog).
*/
//@doc TStatusDlg GUI
#include <owl/pch.h>
# include <owl/commctrl.h>

#if !defined(OWL_STATUSD_H)
#include "statusd.h"
#endif
#include "statusd.rh"

// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TStatusDlg, TDialog)
	EV_WM_CLOSE,
	EV_COMMAND(IDCANCEL, CmClose),
END_RESPONSE_TABLE;

static uint timeMsg[] = {
	IDS_ESTWAIT,
	IDS_ESTLESSMINUTE,
	IDS_ESTABOUTMINUTE,
	IDS_ESTLESSNMINUTE,
	IDS_ESTLESSNHOUR,
	IDS_ESTABOUTHOUR,
	IDS_ESTLESSNHOURS,
};
TStatusDlg::TTimeChecker::TTimeChecker(TModule& module)
:
	Module(module),
	TickCount(0),
	StartTime(0),
	LastValue(0),
	Counter(0),
	eMessage(eWaitTime)
{
}
void
TStatusDlg::TTimeChecker::Reset()
{
	StartTime 	= 0;
	Counter     = 0;
	LastValue   = 0;
	eMessage    = eWaitTime;
}
bool
TStatusDlg::TTimeChecker::Check(char* buf, uint32 count)
{
	if(StartTime == 0){
		StartTime = ::GetTickCount();
		eMessage     = eWaitTime;
    Module.LoadString(timeMsg[eMessage], buf, count);
		return true;
	}

	TickCount = ::GetTickCount();
	Counter++;
	uint32 seconds = (TickCount - StartTime)/(1000*Counter)*(count ? count : 1);
	uint32 curValue = 0;
	uint   value;
	uint eNewMessage;

	// first check for hour
	if((value = (seconds/60 + 15)/60) != 0){
		if (value > 1){
			curValue = value;
			eNewMessage = eLessHours;
		}
		else // if value (hours) = 1 one hour
			eNewMessage = eAboutHour; // curValue already = 0
	}
	// check for a minute
	else if((value = (seconds + 15)/60) != 0){
		// if minutes > 30 then less then hour
		if(value > 30){
			eNewMessage = eLesshour; // curValue already = 0
		}
		// if minutes < 30 but > 1 then show time in minutes
		else if(value > 1){
			curValue = value;
			eNewMessage = eLessMinutes;
		}
		// else about a minute
		else
			eNewMessage = eAboutMinute;
	}
	// else less then a minute
	else
		eNewMessage = eLessMinute; // curValue already = 0

	if(eMessage != eNewMessage || LastValue != curValue){
		eMessage = eNewMessage;
		if(curValue){
			TAPointer<char> buf1 = new char[_MAX_PATH];
	    Module.LoadString(timeMsg[eMessage], buf1, _MAX_PATH);
			wsprintf(buf, buf1, curValue);
		}
		else
	    Module.LoadString(timeMsg[eMessage], buf, count);
		LastValue = curValue;
		return true;
	}
	return false;
}
//------------------------------------------------------------------------------
TStatusDlg::TStatusDlg (TWindow* parent, TResId resId, int length, const char* titles, TModule* module)
:
	TDialog(parent, resId, module),
  Length(length),
  Canceled(false),
  PChecker(*GetModule()),
  CurPos(0)
{
	Buffer = new char[MAX_PATH];
	if(titles)
		SetCaption(titles);
}
//------------------------------------------------------------------------------
TStatusDlg::~TStatusDlg ()
{
	Destroy();
}
//------------------------------------------------------------------------------
//@mfunc SetupWindow
void TStatusDlg::SetupWindow ()
{
	TDialog::SetupWindow();

	SendDlgItemMessage(IDC_STATUSBAR, PBM_SETRANGE, 0, MAKELPARAM(0, Length()));
	SendDlgItemMessage(IDC_STATUSBAR, PBM_SETSTEP, (WPARAM)1, 0l);
}
//------------------------------------------------------------------------------
//@mfunc  Message response function for IDC_CLOSE - do nothing -
// preventing to close windows
void TStatusDlg::CmClose()
{
	//PCancelFlag = true;
  Canceled(true);
}
//------------------------------------------------------------------------------
//@mfunc  Message response function for WM_CLOSE - do nothing -
// preventing to close windows
void TStatusDlg::EvClose()
{
	Canceled(true);
}
//------------------------------------------------------------------------------
//@mfunc Increment Counter and Display message on the screen
void TStatusDlg::Step(char* text)
{
	if(GetHandle()){
		SetDlgItemText(IDC_STATUSTXT1, text);
		SendDlgItemMessage(IDC_STATUSBAR, PBM_STEPIT, 0, 0l);
    // set message for checker
    if(Checker().Check(Buffer, MAX_PATH))
			SetDlgItemText(IDC_STATUSTXT2, Buffer);
	}
}
//------------------------------------------------------------------------------
//@mfunc Display message on the screen
void TStatusDlg::SetMessage(char* text)
{
	if(GetHandle())
		SetDlgItemText(IDC_STATUSTXT1, text);
}
//------------------------------------------------------------------------------
//@mfunc Reset progress bar
void TStatusDlg::Reset()
{
	if(GetHandle()){
		SendDlgItemMessage(IDC_STATUSBAR, PBM_SETPOS, 0, 0l);
    // reset checker
    Checker().Reset();
  }
}
/* ========================================================================== */

